local L = LibStub("AceLocale-3.0"):GetLocale("TrackOMatic", true);

TRACKOMATIC_CURRENCY_COLORS = {
    [L["CURRENCY_HONOR_POINTS"]] = {
        ["Alliance"] = {r = 0.3, g = 0.3, b = 1},
        ["Horde"] = {r = 1, g = 0, b = 0},
    };
    [L["CURRENCY_CONQUEST_POINTS"]] = {
        ["Alliance"] = {r = 0.3, g = 0.3, b = 1},
        ["Horde"] = {r = 1, g = 0, b = 0},
    };
    [L["CURRENCY_JUSTICE_POINTS"]] = {r = 0, g = 0.5, b = 1},
    [L["CURRENCY_DARKMOON_TICKET"]] = {r = 0.9, g = 0.25, b = 0.2},
    [L["CURRENCY_VALOR_POINTS"]] = {r = 0.7, g = 0.6, b = 0.8},
    [L["CURRENCY_DALARAN_JEWELCRAFTERS_TOKEN"]] = {r = 1, g = 0.25, b = 0.5},
    ["default"] = {r = 1, g = 0.85, b = 0}
};

--========================================
-- Add currency with the specified
-- goal amount
--========================================
function TrackOMatic_AddCurrency(name, goalCount)

    goalCount = TrackOMatic_StrToInt(goalCount);
    -- makes sure the goal value entered is a number and that number is greater than 0
    if (goalCount < 1) then
        StaticPopup_Show("TRACKOMATIC_INVALID_GOAL");
        return;
    end

    local currencyName = TrackOMatic_SearchCurrency(name);

    if (currencyName == nil) then
        return;
    end

    if (TrackOMatic_AddToCategory("currency", {id = currencyName, goal = goalCount})) then
        TrackOMatic_Message(string.format(L["TRACKER_ADD_CURRENCY"], currencyName));
    end

end

--========================================
-- Update a currency bar
--========================================
function TrackOMatic_SetCurrencyBar(data)

    local max, value, color;
    local playerFaction = UnitFactionGroup("player");
    local currencyCount = TrackOMatic_GetCurrencyCount(data.id);

    -- set some values
    max = data.goal;
    value = currencyCount;
    if (TRACKOMATIC_CURRENCY_COLORS[data.id]) then
        if (TRACKOMATIC_CURRENCY_COLORS[data.id][playerFaction]) then
            color = TRACKOMATIC_CURRENCY_COLORS[data.id][playerFaction];
        else
            color = TRACKOMATIC_CURRENCY_COLORS[data.id];
        end
    else
        color = TRACKOMATIC_CURRENCY_COLORS["default"];
    end

    local tooltipTitle, tooltipText;
    tooltipTitle = data.id;
    tooltipText = string.format(L["CURRENT_AMOUNT"], currencyCount) .. "\n";
    tooltipText = tooltipText .. string.format(L["GOAL_AMOUNT"], data.goal) .. "\n";
    if (currencyCount < max) then
        tooltipText = tooltipText .. "|cffff8000" .. string.format(L["UNTIL_GOAL"], (max - currencyCount)) .. "|r";
    else
        if (currencyCount > max) then
            tooltipText = tooltipText .. "|cff00ff00" .. string.format(L["PAST_GOAL"], (currencyCount - max)) .. "|r";
        else
            tooltipText = tooltipText .. "|cff00ff00" .. L["GOAL_REACHED"] .. "|r";
        end
    end

    if (data.id == L["CURRENCY_JUSTICE_POINTS"]) then
        if (value >= 375) then
            tooltipText = tooltipText .. "\n\n|cff20a0ff" .. string.format(L["CONVERT_TO_CURRENCY"], "|cffffffff" .. L["CURRENCY_HONOR_POINTS"] .. "|cff20a0ff") .. " x" .. (math.floor(value / 375) * 250);
        end
    elseif (data.id == L["CURRENCY_HONOR_POINTS"]) then
        if (value >= 375) then
            tooltipText = tooltipText .. "\n\n|cff20a0ff" .. string.format(L["CONVERT_TO_CURRENCY"], "|cffffffff" .. L["CURRENCY_JUSTICE_POINTS"] .. "|cff20a0ff") .. " x" .. (math.floor(value / 375) * 250);
        end
    elseif (data.id == L["CURRENCY_VALOR_POINTS"]) then
        if (value >= 250) then
            tooltipText = tooltipText .. "\n\n|cff20a0ff" .. string.format(L["CONVERT_TO_CURRENCY"], "|cffffffff" .. L["CURRENCY_CONQUEST_POINTS"] .. "|cff20a0ff") .. " x" .. (math.floor(value / 250) * 250);
        end
    end

    local levelLabel = "";
    if (TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['show_quantity_currency']) then
        levelLabel = value .. "/" .. max .. " ";
    end
    if (TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['show_percent_currency']) then
        levelLabel = levelLabel .. "(" .. math.floor((value / max) * 100) .. "%)";
    end

    local menu = {
        TrackOMatic_CommonMenuItem("CHANGE_GOAL", function(value) TrackOMatic_ChangeCurrencyGoal(data.id, value); end, data.id),
    };

    local isGlowing = false;
    if (TRACKOMATIC_VARS['config']['currency_default_glow'] and (value >= max)) then
        isGlowing = true;
    end

    return value, max, color, data.id, levelLabel, tooltipTitle, tooltipText, data.id, menu, nil, isGlowing;
end

--========================================
-- Removes a currency from the tracker
--========================================
function TrackOMatic_RemoveCurrency(index, isSilent)
    local data = TrackOMatic_RemoveFromCategory("currency", index);
    if ((not isSilent) and data) then
        TrackOMatic_Message(string.format(L["TRACKER_REMOVE_CURRENCY"], data.id));
    end
end

--========================================
-- Currency search, returns first match
-- by name
--========================================
function TrackOMatic_SearchCurrency(name)

    local crListSize
    crListSize = GetCurrencyListSize();

    for i = 1, crListSize, 1 do
        local crName, isHeader, isExpanded, isUnused, isWatched, count, extraCurrencyType, icon, itemID = GetCurrencyListInfo(i);
        if (isHeader) then
            if (isExpanded ~= true) then
                ExpandCurrencyList(i, 1);
            end
        end
    end

    crListSize = GetCurrencyListSize();

    for i = 1, crListSize, 1 do
        local crName, isHeader, isExpanded, isUnused, isWatched, count, extraCurrencyType, icon, itemID = GetCurrencyListInfo(i);
        if (isHeader ~= true) then
            if (string.sub(string.lower(crName), 0, string.len(name)) == string.lower(name)) then
                return crName;
            end
        end
    end

    return nil;

end

--========================================
-- Returns how much of the specified
-- currency the character has
--========================================
function TrackOMatic_GetCurrencyCount(name)

    local crListSize
    crListSize = GetCurrencyListSize();

    for i = 1, crListSize, 1 do
        local crName, isHeader, isExpanded, isUnused, isWatched, count, extraCurrencyType, icon, itemID = GetCurrencyListInfo(i);
        if (isHeader) then
            if (isExpanded ~= true) then
                ExpandCurrencyList(i, 1);
            end
        end
    end

    crListSize = GetCurrencyListSize();

    for i = 1, crListSize, 1 do
        local crName, isHeader, isExpanded, isUnused, isWatched, count, extraCurrencyType, icon, itemID = GetCurrencyListInfo(i);
        if (isHeader ~= true) then
            if (name == crName) then
                return count;
            end
        end
    end

    return 0;

end

--========================================
-- Changes a currency goal
--========================================
function TrackOMatic_ChangeCurrencyGoal(name, value)
    for index, entry in pairs(TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['categories']['currency'].entries) do
        if (entry.id == name) then
            value = TrackOMatic_StrToInt(value);
            if (value > 0) then
                TRACKOMATIC_VARS[TRACKOMATIC_PROFILE]['categories']['currency'].entries[index].goal = value;
                TrackOMatic_Message(string.format(L["CURRENCY_GOAL_CHANGED"], entry.id, value));
            else
                StaticPopup_Show("TRACKOMATIC_INVALID_GOAL");
                return;
            end
        end
    end
    TrackOMatic_UpdateTracker();
end

--========================================
-- Handler for clicking the add button
-- on the currency tab
--========================================
function TrackOMatic_AddCurrencyButton_OnClick(self)
    local name = TokenFrame.selectedToken;
    TrackOMatic_ShowGoalInput(name, function(value) TrackOMatic_AddCurrency(name, value); end);
end
