local L = LibStub("AceLocale-3.0"):GetLocale("TrackOMatic", true);

TRACKOMATIC_MAX_REP = {
    [8] = 999,
    [7] = 21000,
    [6] = 12000,
    [5] = 6000,
    [4] = 3000,
    [3] = 3000,
    [2] = 3000,
    [1] = 36000
};
TRACKOMATIC_TOTAL_REP = 84999;

-- tables for calculating rep per hour per faction
TRACKOMATIC_STARTING_REP = {};
TRACKOMATIC_REP_FIRSTUPDATE = {};

local questIconNormal = "|TInterface\\GossipFrame\\AvailableQuestIcon:0:0:0:0|t";
local questIconDaily = "|TInterface\\GossipFrame\\DailyQuestIcon:0:0:0:0|t";

--========================================
-- Add a reputation to the tracker
--========================================
function TrackOMatic_AddRep(name)

    local repID, repName = TrackOMatic_GetRepIndex(name);

    if (repID == nil) then
        return;
    end

    if (TrackOMatic_AddToCategory("rep", {id = repName})) then
        TrackOMatic_Message(string.format(L["TRACKER_ADD_REP"], repName));
    end

end

--========================================
-- Update a reputation bar
--========================================
function TrackOMatic_SetRepBar(data)

    local value, max, color, label, levelLabel, tooltipTitle, tooltipText;
    local repIndex = 0;

    -- retrieve the faction info
    local repIndex, factName = TrackOMatic_GetRepIndex(data.id);
    if (repIndex == nil) then
        return;
    end
    local repName, repDesc, standingID, bottomValue, topValue, earnedValue, atWarWith, canToggleAtWar, header, isCollapsed, isWatched = GetFactionInfo(repIndex);

    -- handle rep earn rate
    local repPerHour, timeToExalted, timeToNextLevel;
    if (TRACKOMATIC_STARTING_REP[repName] == nil) then
        TRACKOMATIC_STARTING_REP[repName] = earnedValue;
        TRACKOMATIC_REP_FIRSTUPDATE[repName] = 0;
        repPerHour = 0;
    else
        if (TRACKOMATIC_REP_FIRSTUPDATE[repName] == 0) then
            TRACKOMATIC_REP_FIRSTUPDATE[repName] = GetTime();
            repPerHour = (earnedValue - TRACKOMATIC_STARTING_REP[repName]) / (math.max(1, (GetTime() - TRACKOMATIC_SESSION_START)) / 3600);
        else
            repPerHour = (earnedValue - TRACKOMATIC_STARTING_REP[repName]) / (math.max(1, (GetTime() - TRACKOMATIC_REP_FIRSTUPDATE[repName])) / 3600);
        end
    end
    repPerHour = math.floor(repPerHour)

    -- set some basic values
    local factionStanding = _G["FACTION_STANDING_LABEL" .. standingID];
    local nextStanding = _G["FACTION_STANDING_LABEL" .. math.min(8, standingID + 1)];
    local maxStanding = FACTION_STANDING_LABEL8;
    local repToExalted = TrackOMatic_GetRepToExalted(standingID);
    max = topValue - bottomValue;
    value = earnedValue - bottomValue;
    color = FACTION_BAR_COLORS[standingID];
    label = repName .. " - " .. factionStanding;

    tooltipTitle = string.format(L["STANDING_WITH_FACTION"], factionStanding, repName);
    tooltipText = "";

    if (standingID < 8) then
        local extendedInfo = "";
        if (repName == L["FACTION_TIMBERMAW_HOLD"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationMobKillCounter(20, max - value) .. "\n";
                extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_DEADWOOD_HEADDRESS_FEATHER"], 1, 5, 2000, (max - value)) .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_WINTERFALL_SPIRIT_BEADS"], 1, 5, 2000, (max - value));
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationMobKillCounter(20, repToExalted - value) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_DEADWOOD_HEADDRESS_FEATHER"], 1, 5, 2000, (repToExalted - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_WINTERFALL_SPIRIT_BEADS"], 1, 5, 2000, (repToExalted - value));
                end
            end
        elseif (repName == L["FACTION_SPOREGGAR"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationMobKillCounter(15, max - value) .. "\n";
                -- killing bog lords through honored
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "    " .. L["MOB_BOG_LORD"] .. "\n";
                    extendedInfo = extendedInfo .. "    " .. L["MOB_STARVING_BOG_LORD"] .. "\n";
                    extendedInfo = extendedInfo .. "    " .. L["MOB_STARVING_FUNGAL_GIANT"] .. "\n";
                end
                -- other kills through revered
                extendedInfo = extendedInfo .. "    " .. L["MOB_MARSH_DREDGER"] .. "\n";
                extendedInfo = extendedInfo .. "    " .. L["MOB_MARSH_LURKER"] .. "\n";
                extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                -- below friendly
                if (standingID < 5) then
                    if (standingID == 4) then
                        extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_GLOWCAP"], 1, 10, 750, (max - value)) .. "\n";
                    end
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_BOG_LORD_TENDRIL"], 1, 6, 750, (max - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_MATURE_SPORE_SAC"], 1, 10, 750, (max - value));
                else
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_SANGUINE_HIBISCUS"], 1, 5, 750, (max - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_FERTILE_SPORES"], 1, 6, 750, (max - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_ReputationQuestCounter(L["QUEST_SPOREGGAR_REPEAT"], 750, (max - value), 1);
                end
            end
        elseif (repName == L["FACTION_DARKMOON_FAIRE"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_QUESTS_ANY"], 250, (max - value), 1) .. "\n";
                extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_GAMES_COLLECTIVE"], 1250, (max - value), 2) .. "\n";
                extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_ARTIFACT_SETS"], 2250, (max - value), 1) .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_PROFESSION_QUEST_SETS"], 1500, (max - value), 1);
                extendedInfo = extendedInfo .. "\n\n  " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_MONTHS"], 12500, (max - value));
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_QUESTS_ANY"], 250, (repToExalted - value), 1) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_GAMES_COLLECTIVE"], 1250, (repToExalted - value), 2) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_ARTIFACT_SETS"], 2250, (repToExalted - value), 1) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_PROFESSION_QUEST_SETS"], 1500, (repToExalted - value), 1);
                    extendedInfo = extendedInfo .. "\n\n  " .. TrackOMatic_ReputationQuestCounter(L["DARKMOON_MONTHS"], 12500, (repToExalted - value));
                end
            end
        elseif (repName == L["FACTION_MAGHAR"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationMobKillCounter(10, max - value) .. "\n";
                extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_OBSIDIAN_WARBEADS"], 1, 10, 500, (max - value));
            end
        elseif (repName == L["FACTION_SONS_OF_HODIR"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_RELIC_OF_ULDUAR"], 1, 10, 650, (max - value)) .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_EVERFROST_CHIP"], 1, 1, 350, (max - value)) .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_SONS_OF_HODIR_COMMENDATION_BADGE"], 2, 1, 520, (max - value));
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_RELIC_OF_ULDUAR"], 1, 10, 650, (repToExalted - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_EVERFROST_CHIP"], 1, 1, 350, (repToExalted - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_SONS_OF_HODIR_COMMENDATION_BADGE"], 2, 1, 520, (repToExalted - value));
                end
            end
        elseif (repName == L["FACTION_BLOODSAIL_BUCCANEERS"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationMobKillCounter(25, max - value) .. "\n";
                extendedInfo = extendedInfo .. "    " .. L["MOB_BOOTY_BAY_BRUISER"];
            end
        elseif (repName == L["FACTION_RAVENHOLDT"]) then
            if (standingID < 8) then
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationMobKillCounter(5, max - value) .. "\n";
                end
                extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_HEAVY_JUNKBOX"], 1, 5, 75, (max - value));
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationMobKillCounter(5, repToExalted - value) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_HEAVY_JUNKBOX"], 1, 5, 75, (repToExalted - value));
                end
            end
        elseif ((repName == L["FACTION_BOOTY_BAY"]) or (repName == L["FACTION_GADGETZAN"]) or (repName == L["FACTION_EVERLOOK"]) or (repName == L["FACTION_RATCHET"])) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. string.format(L["MOB_KILLS"], (math.ceil((max - value) / 5)));
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. string.format(L["MOB_KILLS"], (math.ceil((repToExalted - value) / 5)));
                end
            end
        elseif (repName == L["FACTION_CENARION_CIRCLE"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_ENCRYPTED_TWILIGHT_TEXT"], 1, 10, 500, (max - value));
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_ENCRYPTED_TWILIGHT_TEXT"], 1, 10, 500, (repToExalted - value));
                end
            end
        elseif (repName == L["FACTION_DRAGONMAW_CLAN"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["DAILIES_SET"], 1350, (max - value), 2);
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["DAILIES_SET"], 1350, (repToExalted - value), 2);
                end
            end
        elseif ((repName == L["FACTION_DARNASSUS"]) or (repName == L["FACTION_STORMWIND"]) or (repName == L["FACTION_EXODAR"]) or (repName == L["FACTION_IRONFORGE"]) or (repName == L["FACTION_GNOMEREGAN"]) or (repName == L["FACTION_GILNEAS"]) or
                (repName == L["FACTION_ORGRIMMAR"]) or (repName == L["FACTION_THUNDER_BLUFF"]) or (repName == L["FACTION_UNDERCITY"]) or (repName == L["FACTION_SILVERMOON"]) or (repName == L["FACTION_DARKSPEAR_TROLLS"]) or (repName == L["FACTION_BILGEWATER_CARTEL"])) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. string.format(L["DUNGEON_KILLS_NON_TRIVIAL"], (math.ceil((max - value) / math.floor(15 * TrackOMatic_GetReputationMultipliers())))) .. "\n";
                extendedInfo = extendedInfo .. "  " .. string.format(L["BOSS_KILLS_NON_TRIVIAL"], (math.ceil((max - value) / math.floor(300 * TrackOMatic_GetReputationMultipliers()))));
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. string.format(L["DUNGEON_KILLS_NON_TRIVIAL"], (math.ceil((repToExalted - value) / math.floor(15 * TrackOMatic_GetReputationMultipliers())))) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. string.format(L["BOSS_KILLS_NON_TRIVIAL"], (math.ceil((repToExalted - value) / math.floor(300 * TrackOMatic_GetReputationMultipliers()))));
                end
            end
        elseif (repName == L["FACTION_SHATTERED_SUN"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["DAILIES_SET"], 4950, (max - value), 2);
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["DAILIES_SET"], 4950, (repToExalted - value), 2);
                end
            end
        elseif (repName == L["FACTION_THORIUM_BROTHERHOOD"]) then
            if (standingID < 8) then
                extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_FIERY_CORE"], 3, 1, 500, (max - value)) .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_LAVA_CORE"], 3, 1, 500, (max - value)) .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_BLOOD_OF_THE_MOUNTAIN"], 2, 1, 500, (max - value)) .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_CORE_LEATHER"], 1, 2, 500, (max - value)) .. "\n";
                extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_DARK_IRON_ORE"], 1, 10, 500, (max - value));
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "\n\n" .. string.format(L["UNTIL_STANDING"], maxStanding) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. L["TURNINS"] .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_FIERY_CORE"], 3, 1, 500, (repToExalted - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_LAVA_CORE"], 3, 1, 500, (repToExalted - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_BLOOD_OF_THE_MOUNTAIN"], 2, 1, 500, (repToExalted - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_CORE_LEATHER"], 1, 2, 500, (repToExalted - value)) .. "\n";
                    extendedInfo = extendedInfo .. "    " .. TrackOMatic_GetRepTurninInfo(L["ITEM_DARK_IRON_ORE"], 1, 10, 500, (repToExalted - value));
                end
            end
        elseif (repName == L["FACTION_ARGENT_DAWN"]) then
            if (standingID < 8) then
                if (standingID < 7) then
                    extendedInfo = extendedInfo .. "  " .. L["DO_EPL_QUESTS"];
                else
                    extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["QUEST_ARGENTDAWN_REPEAT1"], 1000, (max - value), 2) .. "\n";
                    extendedInfo = extendedInfo .. "  " .. TrackOMatic_ReputationQuestCounter(L["QUEST_ARGENTDAWN_REPEAT2"], 1000, (max - value), 2);
                end
            end
        end
        if (extendedInfo ~= "") then
            tooltipText = string.format(L["UNTIL_STANDING"], nextStanding) .. "\n" .. extendedInfo .. "\n";
        end
    end

    levelLabel = value .. "/" .. max;
    if (standingID < 8) then
        tooltipText = tooltipText .. "\n|cff20a0ff" .. string.format(L["REPUTATION_UNTIL_STANDING"], nextStanding, (max - value));
        if (standingID < 7) then
            tooltipText = tooltipText .. "\n|cff20a0ff" .. string.format(L["REPUTATION_UNTIL_STANDING"], maxStanding, (repToExalted - value));
        end
    end

    if (repPerHour > 0) then
        tooltipText = tooltipText .. "\n|cff20a0ff" .. string.format(L["REPUTATION_PER_HOUR"], repPerHour);

        timeToNextLevel = ((max - value) / repPerHour) * 3600;
        tooltipText = tooltipText .. "\n\n" .. string.format(L["TIME_UNTIL_STANDING"], nextStanding) .. ": " .. TrackOMatic_TimeCounter(timeToNextLevel);

        if (standingID < 8) then
            timeToExalted = ((repToExalted - value) / repPerHour) * 3600;
            if (standingID < 7) then
                tooltipText = tooltipText .. "\n" .. string.format(L["TIME_UNTIL_STANDING"], maxStanding) .. ": " .. TrackOMatic_TimeCounter(timeToExalted);
            end
        end

        -- show earned per hour on bar
        if (TRACKOMATIC_VARS['config']['rep_gain_display'] == 1) then
            levelLabel = levelLabel .. " (" .. repPerHour .. "/hr)";
        -- time until next level
        elseif ((TRACKOMATIC_VARS['config']['rep_gain_display'] == 2) and (standingID < 8)) then
            levelLabel = levelLabel .. " (" .. TrackOMatic_TimeCounter(timeToNextLevel, true) .. ")";
        -- time until exalted
        elseif ((TRACKOMATIC_VARS['config']['rep_gain_display'] == 3) and (standingID < 8)) then
            levelLabel = levelLabel .. " (" .. TrackOMatic_TimeCounter(timeToExalted, true) .. ")";
        end
    else
        tooltipText = tooltipText .. "\n|cff20a0ff" .. string.format(L["REPUTATION_PER_HOUR"], "N/A");
    end

    local menu = {
        TrackOMatic_CommonMenuItem("RESET_SESSION", function() TrackOMatic_ReputationResetSession(repName); end),
    };

    return value, max, color, label, levelLabel, tooltipTitle, tooltipText, repName, menu;
end

--========================================
-- Remove a reputation from the tracker
--========================================
function TrackOMatic_RemoveRep(index, isSilent)
    local data = TrackOMatic_RemoveFromCategory("rep", index);
    if ((not isSilent) and data) then
        TrackOMatic_Message(string.format(L["TRACKER_REMOVE_REP"], data.id));
    end
end

--========================================
-- Get a reputation's index by name
--========================================
function TrackOMatic_GetRepIndex(name)
    local numLines = GetNumFactions();
    local repName, repDesc, standingID, bottomValue, topValue, earnedValue, atWarWith, canToggleAtWar, header, isCollapsed, isWatched;

    for i = 1, numLines, 1 do
        repName, repDesc, standingID, bottomValue, topValue, earnedValue, atWarWith, canToggleAtWar, header, isCollapsed, isWatched = GetFactionInfo(i);
        if (not header) then
            --if (string.find(string.lower(repName), string.lower(name)) ~= nil) then
            if (string.sub(string.lower(repName), 0, string.len(name)) == string.lower(name)) then
                return i, repName;
            end
        end
    end
    return nil;
end

--========================================
-- Generate a turn-in entry for rep
-- tooltips
--========================================
function TrackOMatic_GetRepTurninInfo(itemName, itemQuality, numItemPerTurnin, repPerTurnin, repNeeded)

    local _, questRepMod = TrackOMatic_GetReputationMultipliers();
    local modifiedRepPerTurnin = math.floor(repPerTurnin * questRepMod);

    local turninCount = math.ceil(repNeeded / modifiedRepPerTurnin);
    local _, _, _, itemColor = GetItemQualityColor(itemQuality);

    local hasItems, itemsNeeded, turninStatusColor;

    hasItems = GetItemCount(itemName);
    itemsNeeded = (numItemPerTurnin * turninCount);

    local hasTurnins = math.floor(hasItems / numItemPerTurnin);

    if (hasItems >= itemsNeeded) then
        turninStatusColor = "ffffffff";
    else
        turninStatusColor = "ff808080";
    end

    local itemMult = "";

    if (numItemPerTurnin > 1) then
        itemMult = "|cff00a000x" .. numItemPerTurnin;
    end

    return "|c" .. itemColor .. "[" .. itemName .. "]" .. itemMult .. "|r: " .. turninCount .. " |c" .. turninStatusColor .. "(" .. hasItems .. "/" .. itemsNeeded .. ")|r";

end

--========================================
-- Get total reputation required to
-- reach exalted from current standing
--========================================
function TrackOMatic_GetRepToExalted(standingID)

    local totalToExalted = 0;
    if (standingID < 8) then
        for i = standingID, 7, 1 do
            totalToExalted = totalToExalted + TRACKOMATIC_MAX_REP[i];
        end
    end
    return totalToExalted;

end

--========================================
-- Handler for clicking the button on
-- the reputation detail frame
--========================================
function TrackOMatic_AddRepButton_OnClick(self)
    TrackOMatic_AddRep(ReputationDetailFactionName:GetText());
end

--========================================
-- Calculate GLOBAL reputation gain
-- modifiers (human racial, etc)
--========================================
function TrackOMatic_GetReputationMultipliers()

    local kbonus = 0;
    local qbonus = 0;
    local guildLevel = GetGuildLevel();

    -- check guild perks
    if (guildLevel >= 4) then
        -- +10% rep guild perk
        if (guildLevel >= 12) then
            kbonus = kbonus + 10;
            qbonus = qbonus + 10;
        -- +5% perk
        else
            kbonus = kbonus + 10;
            qbonus = qbonus + 10;
        end
    end

    if (TrackOMatic_UnitHasAura("player", L["BUFF_WOW7THANNIVERSARY"])) then
        kbonus = kbonus + 7;
    end

    local race, raceEn = UnitRace("player");
    if (raceEn == "Human") then
        kbonus = kbonus + 10;
        qbonus = qbonus + 10;
    end

    return (100 + kbonus) / 100, (100 + qbonus) / 100;

end

--========================================
-- Generate a counter for mob kills
-- needed to gain a specified amount of
-- reputation
--========================================
function TrackOMatic_ReputationMobKillCounter(repPerKill, repNeeded)

    local killRepMod = TrackOMatic_GetReputationMultipliers();
    repPerKill = math.floor(repPerKill * killRepMod);

    return string.format(L["MOB_KILLS"], (math.ceil(repNeeded / repPerKill)));

end

--========================================
-- Generate a counter for a quest-based
-- source of reputation in order to gain
-- a specified amount
--========================================
function TrackOMatic_ReputationQuestCounter(label, repGain, repNeeded, questIconType)

    local _, questMod = TrackOMatic_GetReputationMultipliers();
    repGain = math.floor(repGain * questMod);

    questIconType = (questIconType or 0);
    local label = label;
    if (questIconType == 1) then
        label = questIconNormal .. " " .. label;
    elseif (questIconType == 2) then
        label = questIconDaily .. " " .. label;
    end

    return label .. ": " .. (math.ceil(repNeeded / repGain));
    
end

function TrackOMatic_ReputationResetSession(name)
    if (TRACKOMATIC_REP_FIRSTUPDATE[name] or TRACKOMATIC_STARTING_REP[name]) then
        TRACKOMATIC_REP_FIRSTUPDATE[name] = nil;
        TRACKOMATIC_STARTING_REP[name] = nil;
        TrackOMatic_Message(string.format(L["MESSAGE_REP_SESSION_RESET"], name));
        TrackOMatic_UpdateTracker();
    end
end
